mod config;
mod modem;
#[allow(dead_code)]
mod command;

use anyhow;
use std::time::Duration;
use std::thread;
use esp_idf_hal::prelude::*;
use esp_idf_hal::peripherals::Peripherals;
use esp_idf_hal::serial;

fn main() -> anyhow::Result<()> {
    esp_idf_sys::link_patches();

    let dp = Peripherals::take().expect("error taking peripherals");

    // LilyGo TTGO T-Call sim800l board serial pins.
    let serial_rx = dp.pins.gpio26;
    let serial_tx = dp.pins.gpio27;

    let serial_pins = serial::Pins {
        tx: serial_tx,
        rx: serial_rx,
        cts: None,
        rts: None,
    };

    // Create the serial and panic with a message ... if we can't create the serial port, then we
    // can't communicate with the sim800l module, hence we don't run anymore.
    let serial: serial::Serial<serial::UART1, _, _> = serial::Serial::new(
        dp.uart1,
        serial_pins,
        serial::config::Config::default().baudrate(Hertz(115200)),
    )?;

    let (tx, rx) = serial.split();
    let mut mdm = modem::Modem::new(tx, rx);

    let modem_pwrkey = dp.pins.gpio4.into_output()?;
    let modem_rst = dp.pins.gpio5.into_output()?;
    let modem_power = dp.pins.gpio23.into_output()?;

    mdm.init(modem_pwrkey, modem_rst, modem_power)?;

    if !mdm.is_gprs_attached()? {
        let _ = mdm.connect_to_gprs_ap(
            config::A1_GPRS_AP.apn,
            config::A1_GPRS_AP.username,
            config::A1_GPRS_AP.password,
        )?;
    }

    loop {
        if mdm.is_gprs_attached()? {
            let _ = mdm.get_ip_addr()?;

            //println!("connecting to server!");
            //if !mdm.tcp_is_ssl_enabled()? {
            //    let _ = mdm.tcp_ssl_enable()?;
            //}
            if mdm.tcp_is_ssl_enabled()? {
                let _ = mdm.tcp_ssl_disable()?;
            }
            let _ = mdm.tcp_set_quick_mode(false);
            let _ = mdm.tcp_set_manual_receive()?;
            let _ = mdm.tcp_connect("51.158.66.64", 9988)?;
            let _ = mdm.tcp_send("aaaaa")?;
            thread::sleep(Duration::from_millis(1000));
            println!("+++++++++++++++++++++++++++++++++");
            let size = mdm.tcp_receive_reply_len()?;

            let mut reply = vec![0 as u8; size];
            let received_size = mdm.tcp_receive(&mut reply)?;

            println!("expected: {} / received: {}", size, received_size);
            println!("+++++++++++++++++++++++++++++++++");
            println!("REPLY({}) = {}", reply.len(), reply.iter().map(|b| char::from(*b)).collect::<String>());
            println!("+++++++++++++++++++++++++++++++++");
            let _ = mdm.tcp_close_connection()?;
            break;
        }
        println!("!!!!!!!!!!!!GPRS NOT ATTACHED!!!!!!!!!!!");
    }

    Ok(())
}
